import configparser
import datetime

import click
import requests
from requests import adapters

DEF_TIMEOUT = (10, 60)  # (connection time, response time)
CONFIG_FILE = "config.ini"
RETRY_COUNT = 1
DEBUG = True


def read_config(config_file):
    config = configparser.ConfigParser()
    config.read(config_file)
    return config


def setup():
    """Read the configuration file"""

    config = read_config(CONFIG_FILE)

    if 'Authentication' in config:
        rest_api_host = config['Authentication']['RESTAPI-HOST']
        api_key = config['Authentication']['API-KEY']
        tenant_id = config['Authentication']['TENANT-ID']

        return rest_api_host, api_key, tenant_id
    else:
        click.secho('Could not find config data', fg="bright_red")


def get_token(rest_api_host, api_key, tenant_id):
    """Fetches token from IBM SI RestApi service"""

    url = f'{rest_api_host}/restapi/v1/tenants/{tenant_id}/token'
    req_headers = dict()
    req_headers['x-api-key'] = api_key
    req_headers['Accept'] = 'application/json'

    try:
        with requests.Session() as s:
            retry_count = RETRY_COUNT
            if retry_count > 0:
                ha = requests.adapters.HTTPAdapter(max_retries=retry_count)
                s.mount("https://", ha)
            if DEBUG:
                click.secho("Sending request to %s with headers %s (retries: %s)"
                            % (url, req_headers, retry_count), fg="blue")
            r = s.post(url, headers=req_headers, timeout=DEF_TIMEOUT)
            if r.status_code != 201:
                click.secho("Token creation failed with status %d (%s)" % (r.status_code, r.text), fg="bright_red")
                return None
            click.secho("Token creation successful, token valid until %s" % datetime.datetime.fromtimestamp(
                r.json()['result']['expiration'] / 1000.0), fg="green")
            return r.json()
    except requests.exceptions.Timeout:
        click.secho("Request timed out!", fg="bright_red")
        return None
    except Exception as ex:
        click.secho("Unexpected error: %s" % str(ex), fg="bright_red")
        return None


def list_storage_systems(rest_api_host, api_token, tenant_id, storage_type=None):
    """Fetches list of storage systems from IBM SI RestApi service"""

    url = f'{rest_api_host}/restapi/v1/tenants/{tenant_id}/storage-systems'
    if storage_type is not None:
        url = url + f'?storage-type={storage_type}'
    req_headers = dict()
    req_headers['x-api-token'] = api_token
    req_headers['Accept'] = 'application/json'

    try:
        with requests.Session() as s:
            retry_count = RETRY_COUNT
            if retry_count > 0:
                ha = requests.adapters.HTTPAdapter(max_retries=retry_count)
                s.mount("https://", ha)
            if DEBUG:
                click.secho("Sending request to %s with headers %s (retries: %s)"
                            % (url, req_headers, retry_count), fg="blue")
            r = s.get(url, headers=req_headers, timeout=DEF_TIMEOUT)
            if r.status_code != 200:
                click.secho("Failed to fetch list of storage systems with status %d (%s)" % (r.status_code, r.text),
                            fg="bright_red")
                return None
            click.secho("Fetch of list of storage systems successful", fg="green")
            return r.json()
    except requests.exceptions.Timeout:
        click.secho("Request timed out!", fg="bright_red")
        return None
    except Exception as ex:
        click.secho("Unexpected error: %s" % str(ex), fg="bright_red")
        return None


def list_storage_system_volumes(rest_api_host, api_token, tenant_id, system_id, next=None):
    """Fetches list of volumes of a storage system from IBM SI RestApi service"""

    url = f'{rest_api_host}/restapi/v1/tenants/{tenant_id}/storage-systems/{system_id}/volumes' if next is None else next

    req_headers = dict()
    req_headers['x-api-token'] = api_token
    req_headers['Accept'] = 'application/json'

    try:
        with requests.Session() as s:
            retry_count = RETRY_COUNT
            if retry_count > 0:
                ha = requests.adapters.HTTPAdapter(max_retries=retry_count)
                s.mount("https://", ha)
            if DEBUG:
                click.secho("Sending request to %s with headers %s (retries: %s)"
                            % (url, req_headers, retry_count), fg="blue")
            r = s.get(url, headers=req_headers, timeout=DEF_TIMEOUT)
            if r.status_code != 200:
                click.secho(
                    "Failed to fetch list of volumes of a storage system with status %d (%s)" % (r.status_code, r.text),
                    fg="bright_red")
                return None
            click.secho("Fetch of list of volumes of a storage system successful", fg="green")
            return r.json()
    except requests.exceptions.Timeout:
        click.secho("Request timed out!", fg="bright_red")
        return None
    except Exception as ex:
        click.secho("Unexpected error: %s" % str(ex), fg="bright_red")
        return None


def list_tenant_alerts(rest_api_host, api_token, tenant_id, duration=None, severity=None, next=None):
    """Fetches list of alerts for a tenant from IBM SI RestApi service"""

    query_string = ''
    if duration is not None and severity is None:
        query_string += f'?duration={duration}'
    if duration is None and severity is not None:
        query_string += f'?severity={severity}'
    if duration is not None and severity is not None:
        query_string += f'?duration={duration}&severity={severity}'

    url = f'{rest_api_host}/restapi/v1/tenants/{tenant_id}/alerts' + query_string if next is None else next

    req_headers = dict()
    req_headers['x-api-token'] = api_token
    req_headers['Accept'] = 'application/json'

    try:
        with requests.Session() as s:
            retry_count = RETRY_COUNT
            if retry_count > 0:
                ha = requests.adapters.HTTPAdapter(max_retries=retry_count)
                s.mount("https://", ha)
            if DEBUG:
                click.secho("Sending request to %s with headers %s (retries: %s)"
                            % (url, req_headers, retry_count), fg="blue")
            r = s.get(url, headers=req_headers, timeout=DEF_TIMEOUT)
            if r.status_code != 200:
                click.secho(
                    "Failed to fetch list of alerts for a tenant with status %d (%s)" % (r.status_code, r.text),
                    fg="bright_red")
                return None
            click.secho("Fetch of list of alerts for a tenant successful", fg="green")
            return r.json()
    except requests.exceptions.Timeout:
        click.secho("Request timed out!", fg="bright_red")
        return None
    except Exception as ex:
        click.secho("Unexpected error: %s" % str(ex), fg="bright_red")
        return None


def list_tenant_notifications(rest_api_host, api_token, tenant_id, duration=None, severity=None, next=None):
    """Fetches list of notifications for a tenant from IBM SI RestApi service"""

    query_string = ''
    if duration is not None and severity is None:
        query_string += f'?duration={duration}'
    if duration is None and severity is not None:
        query_string += f'?severity={severity}'
    if duration is not None and severity is not None:
        query_string += f'?duration={duration}&severity={severity}'

    url = f'{rest_api_host}/restapi/v1/tenants/{tenant_id}/notifications' + query_string if next is None else next

    req_headers = dict()
    req_headers['x-api-token'] = api_token
    req_headers['Accept'] = 'application/json'

    try:
        with requests.Session() as s:
            retry_count = RETRY_COUNT
            if retry_count > 0:
                ha = requests.adapters.HTTPAdapter(max_retries=retry_count)
                s.mount("https://", ha)
            if DEBUG:
                click.secho("Sending request to %s with headers %s (retries: %s)"
                            % (url, req_headers, retry_count), fg="blue")
            r = s.get(url, headers=req_headers, timeout=DEF_TIMEOUT)
            if r.status_code != 200:
                click.secho(
                    "Failed to fetch list of notifications for a tenant with status %d (%s)" % (r.status_code, r.text),
                    fg="bright_red")
                return None
            click.secho("Fetch of list of notifications for a tenant successful", fg="green")
            return r.json()
    except requests.exceptions.Timeout:
        click.secho("Request timed out!", fg="bright_red")
        return None
    except Exception as ex:
        click.secho("Unexpected error: %s" % str(ex), fg="bright_red")
        return None

